/*
   +----------------------------------------------------------------------+
   | PHP version 4.0                                                      |
   +----------------------------------------------------------------------+
   | Copyright (c) 1997, 1998, 1999, 2000 The PHP Group                   |
   +----------------------------------------------------------------------+
   | This source file is subject to version 2.02 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
   | available at through the world-wide-web at                           |
   | http://www.php.net/license/2_02.txt.                                 |
   | If you did not receive a copy of the PHP license and are unable to   |
   | obtain it through the world-wide-web, please send a note to          |
   | license@php.net so we can mail you a copy immediately.               |
   +----------------------------------------------------------------------+
   | Authors:                                                             |
   |                                                                      |
   +----------------------------------------------------------------------+
 */

#include "php.h"
#include "php_ini.h"
#include "php_smbauth.h"

/* You should tweak config.m4 so this symbol (or some else suitable)
   gets defined.
*/
#if HAVE_SMBAUTH

/* If you declare any globals in php_smbauth.h uncomment this:
ZEND_DECLARE_MODULE_GLOBALS(smbauth)
*/

/* True global resources - no need for thread safety here */
static int le_smbauth;

/* Every user visible function must have an entry in smbauth_functions[].
*/
function_entry smbauth_functions[] = {
	PHP_FE(confirm_smbauth_compiled,	NULL)		/* For testing, remove later. */
	PHP_FE(smbauth,	NULL)		/* For testing, remove later. */
	{NULL, NULL, NULL}	/* Must be the last line in smbauth_functions[] */
};

zend_module_entry smbauth_module_entry = {
	STANDARD_MODULE_HEADER,
	"smbauth",
	smbauth_functions,
	PHP_MINIT(smbauth),
	PHP_MSHUTDOWN(smbauth),
	PHP_RINIT(smbauth),		/* Replace with NULL if there's nothing to do at request start */
	PHP_RSHUTDOWN(smbauth),	/* Replace with NULL if there's nothing to do at request end */
	PHP_MINFO(smbauth),
	STANDARD_MODULE_PROPERTIES
};

#ifdef COMPILE_DL_SMBAUTH
ZEND_GET_MODULE(smbauth)
#endif

/* Remove comments and fill if you need to have entries in php.ini
PHP_INI_BEGIN()
PHP_INI_END()
*/

PHP_MINIT_FUNCTION(smbauth)
{
/* Remove comments if you have entries in php.ini
	REGISTER_INI_ENTRIES();
*/
	return SUCCESS;
}

PHP_MSHUTDOWN_FUNCTION(smbauth)
{
/* Remove comments if you have entries in php.ini
	UNREGISTER_INI_ENTRIES();
*/
	return SUCCESS;
}

/* Remove if there's nothing to do at request start */
PHP_RINIT_FUNCTION(smbauth)
{
	return SUCCESS;
}

/* Remove if there's nothing to do at request end */
PHP_RSHUTDOWN_FUNCTION(smbauth)
{
	return SUCCESS;
}

PHP_MINFO_FUNCTION(smbauth)
{
	php_info_print_table_start();
	php_info_print_table_header(2, "smbauth support", "enabled");
	php_info_print_table_end();

	/* Remove comments if you have entries in php.ini
	DISPLAY_INI_ENTRIES();
	*/
}

/* Remove the following function when you have succesfully modified config.m4
   so that your module can be compiled into PHP, it exists only for testing
   purposes. */

/* Every user-visible function in PHP should document itself in the source */
/* {{{ proto string confirm_smbauth_compiled(string arg)
   Return a string to confirm that the module is compiled in */
PHP_FUNCTION(confirm_smbauth_compiled)
{
	zval **arg;
	int len;
	char string[256];

	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &arg) == FAILURE) {
		WRONG_PARAM_COUNT;
	}

	convert_to_string_ex(arg);

	len = sprintf(string, "Congratulations, you have successfully modified ext/smbauth/config.m4, module %s is compiled into PHP", Z_STRVAL_PP(arg));
	RETURN_STRINGL(string, len, 1);
}

int pam_auth_smb(char *user, char *password, char *server1, char *server2, char *domain);
extern int Valid_User(char *user, char *password, char *server1, char *server2, char *domain);

/* Every user-visible function in PHP should document itself in the source */
/* {{{ proto string confirm_smbauth_compiled(string arg)
   Return a string to confirm that the module is compiled in */
PHP_FUNCTION(smbauth)
{
	zval **arg1;
	zval **arg2;
	zval **arg3;
	zval **arg4;
	zval **arg5;
	int len;
	int r;
	char string[256];

	if (ZEND_NUM_ARGS() != 5 ) {
		WRONG_PARAM_COUNT;
	}

	zend_get_parameters_ex(5, &arg1, &arg2, &arg3, &arg4, &arg5);

	convert_to_string_ex(arg1);
	convert_to_string_ex(arg2);
	convert_to_string_ex(arg3);
	convert_to_string_ex(arg4);
	convert_to_string_ex(arg5);

	r = pam_auth_smb(Z_STRVAL_PP(arg1), Z_STRVAL_PP(arg2), Z_STRVAL_PP(arg3), Z_STRVAL_PP(arg4), Z_STRVAL_PP(arg5));

	if(r == 0)
	{
		len = sprintf(string, "Success");
	}
	else
	{
		len = sprintf(string, "Denied");
	}

	RETURN_STRINGL(string, len, 1);
}
/* }}} */
/* The previous line is meant for emacs, so it can correctly fold and unfold
   functions in source code. See the corresponding marks just before function
   definition, where the functions purpose is also documented. Please follow
   this convention for the convenience of others editing your code.
*/

int pam_auth_smb(char *user, char *password, char *server1, char *server2, char *domain)
{
	int w;
	w=Valid_User(user, password, server1, server2, domain);

        /* Users valid user for return value 0 is success
           1 and 2 indicate Network and protocol failures and
           3 is not logged on
           */

        switch (w)
          {
          case 0 :
            //printf("pam_smb: Correct NT username/password pair\n");
            return 0;
          case 1 :
          case 2 :
            return -1;
          case 3 :
            //printf("pam_smb: NT user not logged on : %s\n", ntname);
            return -1;
          default:
            //printf("pam_smb: Incorrect NT password for username : %s\n", ntname);
            return -1;
          }

        return -1;
}


#endif	/* HAVE_SMBAUTH */


/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 */
